#!/usr/bin/python3

"""
CVE checker for openssl

References:
https://www.openssl.org/news/vulnerabilities.html
http://www.cvedetails.com/vulnerability-list/vendor_id-217/product_id-383/Openssl-Openssl.html

RSS feed: http://www.cvedetails.com/vulnerability-feed.php?vendor_id=217&product_id=383&version_id=&orderby=3&cvssscoremin=0
"""
import re

def openssl_version_from_content(lines):
    """
    Guess a version based on regex of the content for openssl
    """
    new_guess = ""
    pattern1 = re.compile(r"part of OpenSSL ([01]+\.[0-9]+\.[0-9]+[a-z]*) ")
    pattern2 = re.compile(r"OpenSSL ([01]+\.[0-9]+\.[0-9]+[a-z]*) ")

    for line in lines:
        match = pattern1.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2
        match = pattern2.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2

    return new_guess

def contains_openssl(lines):
    """
    Determine if the binary contains openssl
    """
    for line in lines:
        if "part of OpenSSL" in line:
            return 1
        if "openssl.cnf" in line:
            return 1
        if "-DOPENSSL_" in line:
            return 1
    return 0

def get_version(lines, filename):
    """
    Get the version and return it for openssl

    VPkg: openssl, openssl
    """
    version_info = dict()
    if ("libssl.so." in filename) or ("libcrypto.so" in filename):
        version_info["is_or_contains"] = "is"
    elif contains_openssl(lines):
        version_info["is_or_contains"] = "contains"

    if "is_or_contains" in version_info:
        version_info["modulename"] = "openssl"
        version_info["version"] = openssl_version_from_content(lines)

    return version_info



import sys
import abc
import datetime
import json
from os import listdir
from os.path import isfile, join
import os
import sqlite3
import re
import zipfile
import itertools
from collections import namedtuple
if sys.version_info.major == 3:
    from urllib.request import urlopen
else:
    from urllib2 import urlopen

DLPAGE = "https://nvd.nist.gov/download.cfm"
DBNAME = "nvd.vulnerabilities.db"
OUTPUTFILE = "%s_output.csv" % (datetime.datetime.now().strftime("%Y%m%d_%H%M%S"))
CREATE_SYNTAX = '''CREATE TABLE IF NOT EXISTS nvd_data (
                   CVE_Number TEXT,
                   Vendor_Name TEXT,
                   Product_Name TEXT,
                   Exploitability_Score INTEGER,
                   Impact_Score INTEGER,
                   Severity TEXT,
                   version TEXT,
                   PRIMARY KEY(CVE_Number, Vendor_Name, Product_Name))'''
# TODO Windows
DISK_LOCATION_DEFAULT = os.path.join(os.path.expanduser('~'), '.cache',
        'cve-bin-tool')
JSON_FEED = 'https://nvd.nist.gov/vuln/data-feeds#JSON_FEED'
JSON_ZIP = 'https://static.nvd.nist.gov/feeds/json/cve/1.0/'

CVE = namedtuple('CVE', ['number', 'version', 'severity'])

def get_cvelist(output, dbname, json_feed=JSON_FEED, json_zip=JSON_ZIP, **kargs):
    if not os.path.exists(output):
        os.makedirs(output)
    conn = init_database(dbname)

    today = str(datetime.date.today())
    year = str(int(today[:4]))

    # TODO Previous year files will get updated as well.
    r = urlopen(json_feed, **kargs)
    for filename in re.findall("nvdcve-1.0-[0-9]*\.json\.zip", r.read().decode('utf-8')):
        r_file = urlopen(json_zip + filename, **kargs)
        filepath = os.path.join(output, filename)
        if year in filename or not os.path.exists(filepath):
            with open(filepath, 'wb') as f:
                for chunk in r_file:
                    f.write(chunk)
            f.close()
            if year in filename:
                print("Updated current year file "+ filename)
            else:
                print("Creating new file "+ filename)
        else:
            print("Previous year file: " + filename+" already exists")

    #extract_data(conn)
    cve_number, vendor_name, product_name, exploitability_score, impact_score, severity, versions = extract_data(output)
    #print(cve_number[0] + " " + str(exploitability_score[0]) + " " + str(impact_score[0]) + " " + openssl_versions[0])
    store_cve_data(conn, cve_number, vendor_name, product_name, exploitability_score, impact_score, severity, versions)
    display_data(conn)
    conn.close()


def init_database(dbname):
    if not os.path.isfile(dbname):
        print("Database file does not exist. Initializing it")
    conn = sqlite3.connect(dbname)
    c = conn.cursor()
    c.execute(CREATE_SYNTAX)
    c.execute('''CREATE INDEX IF NOT EXISTS vendors ON nvd_data(CVE_Number, Vendor_Name, Product_Name)''')
    return conn


def extract_data(nvddir):
    files = [f for f in listdir(nvddir) if isfile(join(nvddir, f))]
    files.sort()
    cve_number = []
    vendor_name = []
    exploitability_score = []
    impact_score = []
    versions = []
    severity = []
    product_name = []
    for file in files:
        try:
            archive = zipfile.ZipFile(join(nvddir, file), 'r')
            jsonfile = archive.open(archive.namelist()[0])
            cve_dict = json.loads(jsonfile.read().decode("utf-8"))
            # print(len(cve_dict["CVE_Items"]))
            for i, item in enumerate(cve_dict["CVE_Items"]):
                #print(cve_dict["CVE_Items"][i]["cve"]["CVE_data_meta"]["ID"])

                if cve_dict["CVE_Items"][i]["cve"]["affects"]["vendor"]["vendor_data"]:
                    for x, ven in enumerate(cve_dict["CVE_Items"][i]["cve"]["affects"]["vendor"]["vendor_data"]):
                        ver = ''
                        k = 0
                        cve_number.append(cve_dict["CVE_Items"][i]["cve"]["CVE_data_meta"]["ID"])
                        vendor_name.append(cve_dict["CVE_Items"][i]["cve"]["affects"]["vendor"]["vendor_data"][x]["vendor_name"])
                        product_name.append(cve_dict["CVE_Items"][i]["cve"]["affects"]["vendor"]["vendor_data"][x]["product"]["product_data"][0]["product_name"])
                        for j, item in enumerate(cve_dict["CVE_Items"][i]["cve"]["affects"]["vendor"]["vendor_data"][x]["product"]["product_data"][0]["version"]["version_data"]):
                            if k == 0:
                                ver = cve_dict["CVE_Items"][i]["cve"]["affects"]["vendor"]["vendor_data"][x]["product"]["product_data"][0]["version"]["version_data"][j]["version_value"]
                                k = k + 1
                            else:
                                ver = ver + ', ' + cve_dict["CVE_Items"][i]["cve"]["affects"]["vendor"]["vendor_data"][x]["product"]["product_data"][0]["version"]["version_data"][j]["version_value"]
                        versions.append(ver)

                        if 'baseMetricV3' not in cve_dict["CVE_Items"][i]["impact"]:
                            exploitability_score.append(cve_dict["CVE_Items"][i]["impact"]["baseMetricV2"]["exploitabilityScore"])
                            impact_score.append(cve_dict["CVE_Items"][i]["impact"]["baseMetricV2"]["impactScore"])
                            severity.append(cve_dict["CVE_Items"][i]["impact"]["baseMetricV2"]["severity"])
                        else:
                            exploitability_score.append(cve_dict["CVE_Items"][i]["impact"]["baseMetricV3"]["exploitabilityScore"])
                            impact_score.append(cve_dict["CVE_Items"][i]["impact"]["baseMetricV3"]["impactScore"])
                            severity.append(cve_dict["CVE_Items"][i]["impact"]["baseMetricV3"]["cvssV3"]["baseSeverity"])

            jsonfile.close()
        except Exception as e:
            print("Exception in extract_data: " + str(e))
    return cve_number, vendor_name, product_name, exploitability_score, impact_score, severity, versions


def store_cve_data(conn, cve_number, vendor_name, product_name, exploitability_score, impact_score, severity, versions):
    cur = conn.cursor()
    for i, item in enumerate(cve_number):
        try:
            cur.execute('''INSERT OR REPLACE INTO nvd_data(CVE_Number,Vendor_Name,Product_Name,Exploitability_Score,Impact_Score,Severity,version) VALUES(?,?,?,?,?,?,?)''', (cve_number[i], vendor_name[i], product_name[i], exploitability_score[i], impact_score[i], severity[i], versions[i]))
        except Exception as e:
            print("Exception in store_cve_data: " + str(e))
    pk = cur.lastrowid
    conn.commit()
    return pk


def display_data(conn):
    cur = conn.cursor()
    cur.execute('''SELECT * FROM nvd_data where CVE_Number = "CVE-2018-9999"''') #No xml2, xerces, libnss
    rows = cur.fetchall()
    for row in rows:
        print(row)

def get_cvelist_if_stale(nvddir, dbname):
    this_year = datetime.datetime.today().year
    latest_zipfile = os.path.join(nvddir,
                                  "nvdcve-1.0-" + str(this_year) + ".json.zip")

    if not os.path.isfile(latest_zipfile) or (datetime.datetime.today() - datetime.datetime.fromtimestamp(os.path.getmtime(latest_zipfile))) > datetime.timedelta(hours=24):
        get_cvelist(nvddir, dbname)

    if not os.path.isfile(dbname):
        conn = init_database(dbname)
        cve_number, vendor_name, product_name, exploitability_score, impact_score, severity, versions = extract_data(nvddir)
        store_cve_data(conn, cve_number, vendor_name, product_name, exploitability_score, impact_score, severity, versions)
        conn.close()

    else:
        print("Last Update: " + datetime.date.fromtimestamp(os.path.getmtime(dbname)).isoformat())
        print("Local database has been updated in the past 24h.")
        print("New data not downloaded.  Remove old files to force the update.")

    #conn = init_database(dbname)
    #cve_number, vendor_name, product_name, exploitability_score, impact_score, severity, versions = extract_data()
    #store_cve_data(conn, cve_number, vendor_name, product_name, exploitability_score, impact_score, severity, versions)
    #display_data(conn)
    #conn.close()

class NVDSQLite(object):

    def __init__(self, disk_location=DISK_LOCATION_DEFAULT):
        """ Set location on disk where NVD data cache will reside.
        Connect to SQLite database"""
        self.disk_location = disk_location
        self.conn = None

    @property
    def dbname(self):
        """ SQLite datebase file where imported NVD data is stored."""
        return os.path.join(self.disk_location, DBNAME)

    @property
    def nvddir(self):
        """ Directory where NVD zipfiles are stored."""
        return os.path.join(self.disk_location, 'nvd')

    def open(self):
        """ Opens connection to sqlite database."""
        self.conn = sqlite3.connect(self.dbname)

    def close(self):
        """ Closes connection to sqlite database."""
        self.conn.close()
        self.conn = None

    def __enter__(self):
        """ Opens connection to sqlite database."""
        self.open()

    def __exit__(self, exc_type, exc, exc_tb):
        """ Closes connection to sqlite database."""
        self.close()

    def get_cves(self, *vendor_product_pairs):
        """ Get CVEs against a specific version of a package.

        Example:
            nvd.get_cves(('haxx', 'curl'))
            nvd.get_cves(('haxx', 'curl'), ('rubygems', 'curl'))
        """
        cur = self.conn.cursor()
        query = "SELECT CVE_Number, version, Severity FROM nvd_data WHERE " + \
                " OR ".join(itertools.chain(["(Vendor_Name=? AND Product_Name=?)"] * \
                                             len(vendor_product_pairs)))
        if len(vendor_product_pairs) == 1:
            cur.execute(query, vendor_product_pairs[0])
        else:
            cur.execute(query, tuple(itertools.chain(*vendor_product_pairs)))
        return list(map(CVE._make, cur.fetchall()))

    def get_cvelist_if_stale(self):
        """ Update CVEs data from NVD if stale."""
        get_cvelist_if_stale(self.nvddir, self.dbname)



#!/usr/bin/python3

"""
CVE checker for libexpat

References:
http://www.cvedetails.com/vulnerability-list/vendor_id-12037/product_id-22545/Libexpat-Expat.html
http://www.cvedetails.com/vulnerability-list/vendor_id-16735/product_id-39003/Libexpat-Project-Libexpat.html
https://github.com/libexpat/libexpat/blob/master/expat/Changes

RSS feeds:
http://www.cvedetails.com/vulnerability-feed.php?vendor_id=12682&product_id=0&version_id=0&orderby=3&cvssscoremin=0
http://www.cvedetails.com/vulnerability-feed.php?vendor_id=16735&product_id=0&version_id=0&orderby=3&cvssscoremin=0

Easiest way to check CVEs is currently the Changes.txt file.  You can pinpoint the CVEs using grep as follows:
grep 'Release\|CVE' Changes.txt

Which will give you output like...

Release 2.2.5 Tue October 31 2017
Release 2.2.4 Sat August 19 2017
Release 2.2.3 Wed August 2 2017
             #82  CVE-2017-11742 -- Windows: Fix DLL hijacking vulnerability
Release 2.2.2 Wed July 12 2017
Release 2.2.1 Sat June 17 2017
                  CVE-2017-9233 -- External entity infinite loop DoS
(etc.)

"""
import re

def guess_expat_version_from_content(lines):
    """Guesses the expat version from the file contents
    """
    new_guess = ""
    pattern1 = re.compile(r"expat_([012]+\.[0-9]+\.[0-9]+)")
    #pattern2 = re.compile(r"\/expat-([012]+\.[0-9]+\.[0-9]+)-r[0-9]\/")

    for line in lines:
        match = pattern1.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2

    return new_guess

def guess_contains_expat(lines):
    """Tries to determine if a file includes expat
    """
    for line in lines:
        if "reserved prefix (xml) must not be undeclared or bound to another namespace name" in line:
            return 1
        if "cannot change setting once parsing has begun" in line:
            return 1
        if "requested feature requires XML_DTD support in Expat" in line:
            return 1
    return 0

def get_version(lines, filename):
    """returns version information for expat as found in a given file.
    The version info is returned as a tuple:
        [modulename, is_or_contains, version]

    modulename will be expat if expat is found (and blank otherwise)
    is_or_contains idicates if the file is a copy of expat or contains one
    version gives the actual version number

    VPkg: libexpat, expat
    VPkg: libexpat_project, libexpat
    """
    version_info = dict()
    if "libexpat.so." in filename:
        version_info["is_or_contains"] = "is"
    elif guess_contains_expat(lines):
        version_info["is_or_contains"] = "contains"

    if "is_or_contains" in version_info:
        version_info["modulename"] = "expat"
        version_info["version"] = guess_expat_version_from_content(lines)

    return version_info




"""
CVE-bin-tool tests
"""
import os
import sys
import ssl
import shutil
import stat
import tempfile
import unittest
import subprocess
from multiprocessing import Process
from contextlib import contextmanager
if sys.version_info.major == 2:
    from BaseHTTPServer import HTTPServer
    from SimpleHTTPServer import SimpleHTTPRequestHandler
    from urllib2 import URLError
else:
    from urllib.error import URLError
    from http.server import HTTPServer, \
                            SimpleHTTPRequestHandler

from cve_bin_tool.NVDAutoUpdate import NVDSQLite, DBNAME, CREATE_SYNTAX, CVE, \
        get_cvelist

def run_https_server(keyfile, certfile, port):
    httpd = HTTPServer(('127.0.0.1', port), SimpleHTTPRequestHandler)
    ctx = ssl.create_default_context()
    ctx.check_hostname = False
    ctx.verify_mode = ssl.CERT_NONE
    ctx.load_cert_chain(certfile, keyfile=keyfile)
    httpd.socket = ctx.wrap_socket(httpd.socket, server_side=True)
    httpd.serve_forever()

@contextmanager
def test_https_server(tempdir, port):
    keyfile = os.path.join(tempdir, 'key.pem')
    certfile = os.path.join(tempdir, 'cert.pem')
    subprocess.call(['openssl', 'req', '-x509', '-newkey', 'rsa:1024',
                     '-keyout', keyfile, '-out', certfile, '-days', '1',
                     '-nodes', '-sha256', '-subj',
                     '/C=US/ST=Oregon/L=Portland/O=Intel/OU=Org/CN=127.0.0.1'])

    proc = Process(target=run_https_server, args=(keyfile, certfile, port,))
    proc.start()
    try:
        yield certfile
    finally:
        proc.terminate()
        proc.join()

class TestNVD(unittest.TestCase):
    """ Tests NVD file."""

    @classmethod
    def setUpClass(cls):
        cls.tempdir = tempfile.mkdtemp(prefix='cve-bin-tool-')
        cls.nvddir = os.path.join(cls.tempdir, 'nvd')
        cls.dbname = os.path.join(cls.tempdir, DBNAME)

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.tempdir)

    @unittest.skipUnless(os.getenv('LONG_TESTS') == '1', 'Skipping long tests')
    def test_get_cvelist(self):
        get_cvelist(self.nvddir, self.dbname)
        self.assertTrue(os.path.isdir(self.nvddir))
        self.assertTrue(os.path.isfile(self.dbname))

    def test_ssl(self):
        port = 32983
        with test_https_server(self.tempdir, port) as certfile:
            ctx = ssl.create_default_context()
            ctx.load_verify_locations(cafile=certfile)
            get_cvelist(self.nvddir, self.dbname,
                        json_feed='https://127.0.0.1:%d/' % (port,),
                        json_zip='https://127.0.0.1:%d/' % (port,),
                        context=ctx)

    def test_ssl_failure(self):
        port = 32983
        with test_https_server(self.tempdir, port) as certfile:
            with self.assertRaises(URLError):
                get_cvelist(self.nvddir, self.dbname,
                            json_feed='https://127.0.0.1:%d/' % (port,),
                            json_zip='https://127.0.0.1:%d/' % (port,))

class TestNVDSQLite(unittest.TestCase):
    """ Tests NVDSQLite class."""

    @classmethod
    def setUpClass(cls):
        cls.tempdir = tempfile.mkdtemp(prefix='cve-bin-tool-')
        cls.nvd = NVDSQLite(cls.tempdir)

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.tempdir)

    def test_dbname(self):
        self.assertEqual(self.nvd.dbname, os.path.join(self.tempdir, DBNAME))

    def test_nvddir(self):
        self.assertEqual(self.nvd.nvddir, os.path.join(self.tempdir, 'nvd'))

    def test_open_close(self):
        with self.nvd:
            self.assertNotEqual(self.nvd.conn, None)
        self.assertEqual(self.nvd.conn, None)

    def test_file_permissions(self):
        # Since we are pointed at a tempdir make sure the dbfile in the tempdir
        # is created before checking permissions by using with nvd.
        with self.nvd:
            # User should be able to read/write the dbfile
            self.assertNotEqual(os.stat(self.nvd.dbname).st_mode & stat.S_IRUSR, 0)
            self.assertNotEqual(os.stat(self.nvd.dbname).st_mode & stat.S_IWUSR, 0)

            # Others should not be able to write/execute the dbfile
            self.assertEqual(os.stat(self.nvd.dbname).st_mode & stat.S_IXOTH, 0)
            self.assertEqual(os.stat(self.nvd.dbname).st_mode & stat.S_IWOTH, 0)

    def test_get_cves(self):
        check = [('CVE-001', 'example0', 'app0'),
                 ('CVE-002', 'example1', 'app1')]
        with self.nvd:
            c = self.nvd.conn.cursor()
            c.execute(CREATE_SYNTAX)
            for cve in check:
                c.execute('''INSERT INTO nvd_data(
                             CVE_Number, Vendor_Name, Product_Name)
                             VALUES(?,?,?)''', cve)
            self.nvd.conn.commit()
            # Check that single select works
            for cve in check:
                found = self.nvd.get_cves(cve[1:])
                self.assertEqual(len(found), 1)
                self.assertEqual(found[0].number, cve[0])
            # Check that multi select works
            found = self.nvd.get_cves(*[cve[1:] for cve in check])
            self.assertEqual(len(found), 2)



#!/usr/bin/python3

"""
CVE checker for libxerces

References:
http://www.cvedetails.com/vulnerability-list/vendor_id-45/product_id-4103/Apache-Xerces-c-.html

RSS feed: http://www.cvedetails.com/vulnerability-feed.php?vendor_id=45&product_id=4103&version_id=&orderby=2&cvssscoremin=0
"""
import re

def guess_xerces_version_from_content(lines):
    """Guesses the xerces version from the file contents
    """
    new_guess = ""
    pattern1 = re.compile(r"\/xerces-c-src_([0-9]+_[0-9]+_[0-9]+)\/")

    for line in lines:
        match = pattern1.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2

    return new_guess.replace('_', '.')

def guess_contains_xerces(lines):
    """Tries to determine if a file includes xerces
    """
    for line in lines:
        if "xerces-c-src_" in line:
            return 1
    return 0

def get_version(lines, filename):
    """returns version information for xerces as found in a given file.
    The version info is returned as a tuple:
        [modulename, is_or_contains, version]

    modulename will be xerces if xerces is found (and blank otherwise)
    is_or_contains idicates if the file is a copy of xerces or contains one
    version gives the actual version number

    VPkg: apache, xerces-c
    """
    version_info = dict()
    if "libxerces-c.so" in filename or "libxerces-c-3.1.so" in filename:
        version_info["is_or_contains"] = "is"
    elif guess_contains_xerces(lines):
        version_info["is_or_contains"] = "contains"

    if "is_or_contains" in version_info:
        version_info["modulename"] = "xerces"
        version_info["version"] = guess_xerces_version_from_content(lines)

    return version_info



#!/usr/bin/python3

"""
CVE checker for libtiff

http://www.cvedetails.com/vulnerability-list/vendor_id-2224/product_id-3881/Libtiff-Libtiff.html

RSS feed: http://www.cvedetails.com/vulnerability-feed.php?vendor_id=2224&product_id=3881&version_id=&orderby=3&cvssscoremin=0
"""

import re

def guess_tiff_version_from_content(lines):
    """Guesses the tiff version from the file contents
    """
    new_guess = ""
    pattern1 = re.compile(r"LIBTIFF, Version ([0-9]+\.[0-9]+\.[0-9]+)")

    for line in lines:
        match = pattern1.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2

    return new_guess

def guess_contains_tiff(lines):
    """Tries to determine if a file includes tiff
    """
    for line in lines:
        if "LIBTIFF, Version " in line:
            return 1
        if "Unknown TIFF resolution unit %d ignored" in line:
            return 1
        if "TIFF directory is missing required \"StripByteCounts\" field, calculating from imagelength" in line:
            return 1
    return 0

def get_version(lines, filename):
    """returns version information for tiff as found in a given file.
    The version info is returned as a tuple:
        [modulename, is_or_contains, version]

    modulename will be tiff if tiff is found (and blank otherwise)
    is_or_contains idicates if the file is a copy of tiff or contains one
    version gives the actual version number

    VPkg: libtiff, libtiff
    """
    version_info = dict()
    if "libtiff.so." in filename:
        version_info["is_or_contains"] = "is"
    elif guess_contains_tiff(lines):
        version_info["is_or_contains"] = "contains"

    if "is_or_contains" in version_info:
        version_info["modulename"] = "tiff"
        version_info["version"] = guess_tiff_version_from_content(lines)

    return version_info




"""
CVE-bin-tool tests
"""
import os
import re
import subprocess
import sys
import unittest

from cve_bin_tool.NVDAutoUpdate import NVDSQLite
from cve_bin_tool.cli import Scanner

BINARIES_PATH = os.path.join(os.path.abspath(os.path.dirname(__file__)),
                             'binaries')

class TestScanner(unittest.TestCase):
    """Runs a series of tests against our "faked" binaries.

    The faked binaries are very small c files containing the same string signatures we use
    in the cve-bin-tool.  They should trigger results as if they contained the library and
    version specified in the file name.

    At this time, the tests work only in python3.
    """

    @classmethod
    def setUpClass(cls):
        # Run makefile to build faked binaries (in python 3 or 2)
        subprocess.call(["make", "clean"], cwd=BINARIES_PATH)
        subprocess.call(["make", "all"], cwd=BINARIES_PATH)
        # Instantiate the NVD database
        cls.nvd = NVDSQLite()
        cls.nvd.get_cvelist_if_stale()
        # Instantiate a scanner
        cls.scanner = Scanner(cls.nvd)

    def setUp(self):
        self.nvd.open()

    def tearDown(self):
        self.nvd.close()

    def scan_file(self, filename):
        return self.scanner.scan_file(os.path.join(BINARIES_PATH, filename))

    def test_vendor_package_pairs(self):
        pairs = list(self.scanner.vendor_package_pairs('\n'.join([
            'VPkg: haxx, curl',
            'VPkg: feed, face'
            ])))
        self.assertTrue(pairs)
        self.assertIn('haxx', dict(pairs))
        self.assertIn('curl', dict(map(lambda x: x[::-1], pairs)))
        self.assertIn('feed', dict(pairs))
        self.assertIn('face', dict(map(lambda x: x[::-1], pairs)))

    def test_cannot_open_file(self):
        with self.assertRaises(ValueError):
            self.scan_file('non-existant-file')

    def _binary_test(self, binary, package, version, are_in, not_in):
        """ Helper function to scan a binary and check that it contains certain
        cves for a version and doesn't contain others."""
        # Run the scan
        cves = self.scan_file(binary)
        # Make sure the package and version are in the results
        self.assertIn(package, cves)
        self.assertIn(version, cves[package])
        # Test for CVEs known in this version
        for ensure_in in are_in:
            self.assertIn(ensure_in, cves[package][version])
        # Test for a CVE that is not in this version
        for ensure_out in not_in:
            self.assertNotIn(ensure_out, cves[package][version])

    def test_curl_7_57_0(self):
        """Scanning test-curl-7.57.0.out"""
        self._binary_test('test-curl-7.57.0.out', 'curl', '7.57.0', [
            'CVE-2018-1000122',
            'CVE-2018-1000121',
            'CVE-2018-1000120',
            'CVE-2018-1000007',
            # NOTE Missing from NVD as of 1/14/2019, reported to NVD
            # 'CVE-2018-1000005',
            ], [
            'CVE-2017-8818',
            ])

    def test_expat_2_0_1(self):
        """Scanning test-expat-2.0.1.out"""
        # TODO Commented out issues have to do with less than X.X.X version
        # issue with NVD. Issue #29
        self._binary_test('test-expat-2.0.1.out', 'expat', '2.0.1', [
            # Check for issues specific to expat 2.0.1
            "CVE-2012-1147",
            # "CVE-2009-3720",
            # "CVE-2009-3560",
            "CVE-2012-1148",
            "CVE-2012-0876",

            # Check for other issues from more recent versions
            # 2.1
            "CVE-2016-0718",
            # "CVE-2016-4472",
            # "CVE-2016-5300",
            # "CVE-2012-6702",
            # "CVE-2015-1283",

            # 2.2
            # "CVE-2017-9233",
            # "CVE-2016-9063",
            "CVE-2016-0718",
            # "CVE-2017-11742",
            ], [
            'CVE-blahblah',
            ])

    def test_node_9_3_0(self):
        """Scanning test-node-9.3.0.out"""
        cves = self.scan_file("test-node-9.3.0.out")
        self._binary_test('test-node-9.3.0.out', 'node', '9.3.0', [
            # Check for known cves in 9.3.0
            "CVE-2017-15896",
            "CVE-2017-15897",
            ], [
            # Check to make sure an older CVE from 8.7.0 isn't included
            "CVE-2017-14919",
            ])

    def test_nss_3_35(self):
        """Scanning test-nss-3.55.out"""
        self._binary_test('test-nss-3.35.out', 'nss', '3.35', [
            # Check for known cves in 3.35
            "CVE-2017-11695",
            "CVE-2017-11696",
            "CVE-2017-11697",
            "CVE-2017-11698",
            ], [
            # Check to make sure an older CVE from 3.30.1 isn't included
            "CVE-2017-7502",
            ])

    def test_openssl_1_0_2g(self):
        """Scanning test-openssl-1.0.2g.out"""
        self._binary_test('test-openssl-1.0.2g.out', 'openssl', '1.0.2g', [
            # Check for known cves in this version
            "CVE-2016-2107",
            "CVE-2016-2105",
            "CVE-2016-2106",
            "CVE-2016-2109",
            "CVE-2016-2176",
            ], [
            # Check to make sure an older CVE isn't included
            "CVE-2016-0800",
            ])

    def test_png_1_6_26(self):
        """Scanning test-png-1.6.26.out"""
        self._binary_test('test-png-1.6.26.out', 'png', '1.6.26', [
            # Check for known cves in this version
            "CVE-2016-10087",
            ], [
            # Check to make sure an older CVE isn't included
            "CVE-2015-8126",
            ])

    def test_tiff_4_0_9(self):
        """Scanning test-tiff-4.0.9.out"""
        self._binary_test('test-tiff-4.0.9.out', 'tiff', '4.0.9', [
            # Check for known cves in this version
            "CVE-2017-18013",
            "CVE-2017-17942",
            "CVE-2017-17095",
            "CVE-2018-5784",
            "CVE-2018-7456",
            "CVE-2018-8905",
            ], [
            # Check to make sure an older CVE isn't included
            "CVE-2017-17973",
            ])

    def test_xerces_3_1_1(self):
        """Scanning test-xerces-3_1_1.out"""
        self._binary_test('test-xerces-3_1_1.out', 'xerces', '3.1.1', [
            # Check for known cves in this version
            "CVE-2015-0252",
            ], [
            # Check to make sure an older CVE isn't included
            "CVE-EOL-DONOTUSE",
            ])

    def test_xml2_2_9_0(self):
        """Scanning test-xml2-2.9.0.out"""
        self._binary_test('test-xml2-2.9.0.out', 'xml2', '2.9.0', [
            # Check for known cves in this version
            "CVE-2013-0338",
            "CVE-2013-1969",
            ], [
            # Check to make sure an older CVE isn't included
            "CVE-2011-1944",
            ])

    def test_zlib_1_2_8(self):
        """Scanning test-zlib-1.2.8.out"""
        self._binary_test('test-zlib-1.2.8.out', 'zlib', '1.2.8', [
            # Check for known cves in this version
            "CVE-2016-9843",
            "CVE-2016-9842",
            "CVE-2016-9841",
            "CVE-2016-9840",
            ], [
            # Check to make sure an older CVE isn't included
            "CVE-2005-2096",
            ])



#!/usr/bin/python3
# pylint: disable=invalid-name

"""
This tool scans for a number of common, vulnerable open source components
(openssl, libpng, libxml2, expat and a few others) to let you know if your
system includes common libraries with known vulnerabilities.  It emits a list
of CVE numbers that may be relevant to your binary based on the versions.  It
cannot detect backported fixes.
"""

# Python 2 compatibility stuff
from __future__ import print_function
import sys
import os
import csv
import platform
import subprocess
import re
import logging
import argparse
import pkg_resources
from collections import namedtuple

from .util import DirWalk
from .extractor import Extractor
from .NVDAutoUpdate import get_cvelist, NVDSQLite
from .log import LOGGER

Checker = namedtuple('Checker',
                     ['name', 'get_versions', 'vendor_product_pairs'])

class Scanner(object):
    """"Scans files for CVEs using CVE checkers"""

    CHECKER_ENTRYPOINT = 'cve_bin_tool.checker'

    def __init__(self, nvd, checkers=None, verbose=True, logger=None):
        if logger is None:
            logger = LOGGER.getChild(self.__class__.__name__)
        # Load checkers if not given
        if checkers is None:
            checkers = self.load_checkers()
        self.nvd = nvd
        self.checkers = checkers
        self.logger = logger
        self.verbose = verbose
        self.all_cves = {}
        self.files_with_cve = 0
        self.logger.info('Checkers loaded: %s',
                         ', '.join(self.checkers.keys()))

    @staticmethod
    def vendor_package_pairs(docstring):
        """Generator yielding each instance of a vendor name, package name pair
        in a checkers docstring"""
        for line in docstring.split('\n'):
            if line.strip().startswith('VPkg:'):
                yield tuple(map(lambda x: x.strip(),
                                line.replace('VPkg:', '').split(',')))

    @classmethod
    def load_checkers(cls):
        """ Loads CVE checkers """
        checkers = dict(map(lambda checker: (checker.name, checker.load()),
                            pkg_resources.iter_entry_points(cls.CHECKER_ENTRYPOINT)))
        for checker_name, checker in checkers.items():
            vendor_package_pairs = list(cls.vendor_package_pairs(checker.__doc__))
            if not vendor_package_pairs:
                raise ValueError('Docstring of %r checker' % (checker_name) + \
                                 ' does not define a vendor package pair')
            checkers[checker_name] = (checker, vendor_package_pairs)
        return checkers

    def get_cves(self, vendor_package_pairs, vers):
        """Returns a list of cves affecting a given version of tiff
        """
        cves_by_version = dict()
        for row in self.nvd.get_cves(*vendor_package_pairs):
            for ver in [ver.strip() for ver in row.version.split(',')]:
                if not ver in cves_by_version:
                    cves_by_version[ver] = {}
                cves_by_version[ver][row.number] = row
        # `-` version is all versions are affected. Aka no patch
        grab_all = {}
        grab_all.update(cves_by_version.get('-', {}))
        grab_all.update(cves_by_version.get(vers, {}))
        return grab_all

    def scan_file(self, filename):
        """Scans a file to see if it contains any of the target libraries,
        and whether any of those contain CVEs"""

        self.logger.debug('Scanning file: %r', filename)

        # step 1: check if it's an ELF binary file
        o = subprocess.check_output(['file', filename])
        if sys.version_info.major == 3:
            o = o.decode('utf-8')

        if 'cannot open' in o:
            raise ValueError(o)

        if ("LSB shared" not in o) and ("LSB executable" not in o) and ("PE32 executable" not in o) and ("PE32+ executable" not in o) and ("Mach-O" not in o):
            return

        o = subprocess.check_output(["strings", filename])
        if sys.version_info.major == 3:
            o = o.decode('utf-8')
        lines = o.split("\n")

    #tko
        for checker_name, (get_version, vendor_package_pairs) in self.checkers.items():
            result = get_version(lines, filename)
            if "is_or_contains" in result:
                modulename, version = result["modulename"], result["version"]
                found_cves = self.get_cves(vendor_package_pairs, version)
                if len(found_cves.keys()):
                    self.files_with_cve = self.files_with_cve + 1
                if not modulename in self.all_cves:
                    self.all_cves[modulename] = {}
                self.all_cves[modulename][version] = found_cves
                if self.verbose:
                    print(filename, result["is_or_contains"], modulename,
                          version)
                    if len(found_cves.keys()):
                        print("Known CVEs in version " + version)
                        print(', '.join(found_cves.keys()))
        return self.all_cves

    def affected(self):
        """ Returns list of module name and version tuples identified from
        scan"""
        return [(modulename, version) \
                for modulename, versions in self.all_cves.items() \
                for version in versions.keys()]

class LogAction(argparse.Action):
    """ Argparse action for selecting logging level."""

    def __call__(self, parser, namespace, value, option_string=None):
        """ Turns string into logging level definition."""
        setattr(namespace, self.dest,
                getattr(logging, value.upper(), logging.WARNING))

def scan_and_or_extract_file(scanner, ectx, walker, should_extract, filepath):
    """ Runs extraction if possible and desired otherwise scans."""
    if ectx.can_extract(filepath):
        if not should_extract:
            print("%s is an archive. Pass " % (filepath,) + \
                  "-x option to auto-extract")
            return
        for filename in walker([ectx.extract(filepath)]):
            scan_and_or_extract_file(scanner, ectx, walker, should_extract,
                                     filename)
    else:
        scanner.scan_file(filepath)

def output_cves(outfile, modules, include_details=False):
    writer = csv.writer(outfile)
    for modulename, versions in modules.items():
        for version, cves in versions.items():
            for number, cve in cves.items():
                row = [modulename, version, cve.number, cve.severity]
                if include_details:
                    # TODO Include description in import from NVD jsons
                    # row.append(cve.description)
                    pass
                writer.writerow(row)

def main(argv=sys.argv, outfile=sys.stdout):
    """ Scan a binary file for certain open source libraries that may have CVEs """
    usage = """Usage: cve-binary-tools.py <path to directory>

    Possible output levels:
    -v (verbose): print scan results as they're found
       (regular): print only final summary
    -q (quiet):   suppress all output but exit with error
                  number indicating number of files with CVE

    Other options:
    -x (extract): Autoextract compressed files
    """

    if len(argv) <= 1:
        print(usage)
        return 0

    parser = argparse.ArgumentParser()
    parser.add_argument("directory",
                        help="directory to scan")
    parser.add_argument('-x', "--extract", action="store_true",
                        help="autoextract compressed files")
    parser.add_argument('-v', "--verbose", action="store_true",
                        help="details on found issues as script runs")
    parser.add_argument('-q', "--quiet", action="store_true",
                        help="suppress output")
    parser.add_argument('-l', "--log", help="log level", dest='log_level',
                        action=LogAction,
                        choices=['debug', 'info', 'warning', 'error', 'critical'])

    args = parser.parse_args(argv[1:])

    logging.basicConfig(level=args.log_level)

    if platform.system() != "Linux":
        warning_nolinux = """
                          **********************************************
                          Warning: this utility was developed for Linux.
                          You may need to install additional utilities
                          to use it on other operating systems.
                          **********************************************
                          """
        print(warning_nolinux)

    if not os.path.isfile(args.directory) and not os.path.isdir(args.directory):
        print("Error: directory/file invalid")
        print(usage)
        return 0

    exclude_folders = ['.git']
    walker = DirWalk(
        #pattern='*.*',
        #folder_include_pattern='*',
        folder_exclude_pattern=';'.join(exclude_folders),
        #file_exclude_pattern=';'.join(exclude_files)
    ).walk

    # Update CVE database
    if not args.quiet:
        print("Connecting to NVD database and extracting the CVE list ... " + \
              "Please hold on.. This will take few minutes... ")
    nvd = NVDSQLite()
    nvd.get_cvelist_if_stale()

    # Close database when done
    with nvd:
        extractor = Extractor()
        scanner = Scanner(nvd, verbose=args.verbose)

        with extractor() as ectx:
            if os.path.isdir(args.directory):
                for filepath in walker([args.directory]):
                    scan_and_or_extract_file(scanner, ectx, walker,
                                             args.extract, filepath)
            elif os.path.isfile(args.directory):
                scan_and_or_extract_file(scanner, ectx, walker,
                                         args.extract, args.directory)

        if (not args.quiet) and scanner.files_with_cve > 0:
            print("")
            print("Overall CVE summary: ")
            print("There are", scanner.files_with_cve, "files with known CVEs detected")
            affected_string = ', '.join(map(lambda module_version:
                ' '.join(module_version), scanner.affected()))
            print("Known CVEs in %s:" % (affected_string,))
            output_cves(outfile, scanner.all_cves, include_details=args.verbose)

        # Use the number of files with known cves as error code
        # as requested by folk planning to automate use of this script.
        # If no files found, then the program exits cleanly.
        return scanner.files_with_cve


if __name__ == '__main__':
    sys.exit(main())




#!/usr/bin/python3

"""
CVE checker for libxml2

References:
http://www.cvedetails.com/vulnerability-list/vendor_id-1962/product_id-3311/Xmlsoft-Libxml2.html

RSS feed: http://www.cvedetails.com/vulnerability-feed.php?vendor_id=1962&product_id=3311&version_id=&orderby=2&cvssscoremin=0
"""
import re

def guess_xml2_version_from_content(lines):
    """Guesses the xml2 version from the file contents
    """
    new_guess = ""
    pattern1 = re.compile(r"\/libxml2\-([0-9]+\.[0-9]+\.[0-9]+)\/")
    pattern2 = re.compile(r"\\libxml2\-([0-9]+\.[0-9]+\.[0-9]+)\\")

    for line in lines:
        match = pattern1.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2

        match = pattern2.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2
        if line == "20901":
            new_guess = "2.9.1"
        if line == "20902":
            new_guess = "2.9.2"
        if line == "20903":
            new_guess = "2.9.3"
        if line == "20904":
            new_guess = "2.9.4"
    return new_guess

def guess_contains_xml2(lines):
    """Tries to determine if a file includes xml2
    """
    for line in lines:
        if "Internal error, xmlCopyCharMultiByte 0x%X out of bound" in line:
            return 1
        if "xmlNewElementContent : name != NULL !" in line:
            return 1
        if "xmlRelaxNG: include %s has a define %s but not the included grammar" in line:
            return 1
    return 0

def get_version(lines, filename):
    """returns version information for xml2 as found in a given file.
    The version info is returned as a tuple:
        [modulename, is_or_contains, version]

    modulename will be xml2 if xml2 is found (and blank otherwise)
    is_or_contains idicates if the file is a copy of xml2 or contains one
    version gives the actual version number

    VPkg: xmlsoft, libxml2
    """
    version_info = dict()
    if "libxml2.so." in filename:
        version_info["is_or_contains"] = "is"
    elif guess_contains_xml2(lines):
        version_info["is_or_contains"] = "contains"

    if "is_or_contains" in version_info:
        version_info["modulename"] = "xml2"
        version_info["version"] = guess_xml2_version_from_content(lines)

    return version_info




"""
CVE-bin-tool CLI tests
"""
import os
import re

from cve_bin_tool.cli import main
from cve_bin_tool.extractor import Extractor

from .test_definitions import TempDirTest, \
                              download_file, \
                              CURL_7_20_0_RPM, \
                              CURL_7_20_0_URL

class TestCLI(TempDirTest):
    """ Tests the CVE Bin Tool CLI"""

    @classmethod
    def setUpClass(cls):
        super(TestCLI, cls).setUpClass()
        download_file(CURL_7_20_0_URL,
                      os.path.join(cls.tempdir, CURL_7_20_0_RPM))

    def test_extract_curl_7_20_0(self):
        """Scanning curl-7.20.0"""
        self.assertNotEqual(main(['cve-bin-tool', '-l', 'debug', '-x',
                                  self.tempdir]), 0)

    def test_binary_curl_7_20_0(self):
        with Extractor()() as ectx:
            extracted_path = ectx.extract(os.path.join(self.tempdir,
                                                       CURL_7_20_0_RPM))
            self.assertNotEqual(main(['cve-bin-tool', '-l', 'debug',
                                      os.path.join(extracted_path, 'usr', 'bin',
                                                   'curl')]), 0)

    def test_no_extraction(self):
        self.assertEqual(main(['cve-bin-tool',
                               os.path.join(self.tempdir, CURL_7_20_0_RPM)]),
                               0)

    def test_usage(self):
        self.assertEqual(main(['cve-bin-tool']), 0)

    def test_invalid_file_or_directory(self):
        self.assertEqual(main(['cve-bin-tool', 'non-existant']), 0)



#!/usr/bin/python3

"""
CVE checker for zlib
--------
References:
http://www.cvedetails.com/vulnerability-list/vendor_id-72/product_id-1820/GNU-Zlib.html
https://zlib.net/ChangeLog.txt

RSS feed: http://www.cvedetails.com/vulnerability-feed.php?vendor_id=72&product_id=1820&version_id=&orderby=2&cvssscoremin=0
"""
import re

def guess_zlib_version_from_content(lines):
    """Guesses the zlib version from the file contents
    """
    new_guess = ""
    pattern1 = re.compile(r"deflate ([01]+\.[0-9]+\.[0-9]+) ")
    pattern2 = re.compile(r"inflate ([01]+\.[0-9]+\.[0-9]+) ")

    for line in lines:
        match = pattern1.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2
        match = pattern2.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2

    return new_guess

def guess_contains_zlib(lines):
    """Tries to determine if a file includes zlib
    """
    for line in lines:
        if "deflate" in line and "Copyright 1995-2005 Jean-loup Gailly" in line:
            return 1
        if "Copyright 1995-2005 Mark Adler" in line and "inflate" in line:
            return 1
        if "too many length or distance symbols" in line:
            return 1
    return 0

def get_version(lines, filename):
    """returns version information for zlib as found in a given file.
    The version info is returned as a tuple:
        [modulename, is_or_contains, version]

    modulename will be zlib if zlib is found (and blank otherwise)
    is_or_contains idicates if the file is a copy of zlib or contains one
    version gives the actual version number

    VPkg: gnu, zlib
    """

    version_info = dict()
    if "libz.so." in filename:
        version_info["is_or_contains"] = "is"
    elif guess_contains_zlib(lines):
        version_info["is_or_contains"] = "contains"

    if "is_or_contains" in version_info:
        version_info["modulename"] = "zlib"
        version_info["version"] = guess_zlib_version_from_content(lines)

# FIXME: the original code has a version check as follows that we might
# need to include here.
#     if is_zlib == 1 and (zlib_version not in "1.2.8") and (zlib_version not in "1.2.5") and (zlib_version not in "1.2.3"):
#           # print cve info

    return version_info



#!/usr/bin/python3

"""
CVE checker for node.js

References:
http://www.cvedetails.com/vulnerability-list/vendor_id-12113/Nodejs.html

RSS feed: http://www.cvedetails.com/vulnerability-feed.php?vendor_id=12113&product_id=0&version_id=&orderby=3&cvssscoremin=0
"""
import re

def guess_node_version_from_content(lines):
    """Guesses the node version from the file contents
    """
    new_guess = ""
    pattern1 = re.compile(r"https\:\/\/nodejs.org\/download\/release\/v([0-9]+\.[0-9]+\.[0-9]+)\/")
    pattern2 = re.compile(r"node v([0-9]+\.[0-9]+\.[0-9]+)")

    for line in lines:
        match = pattern1.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2
        match = pattern2.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2

    return new_guess

def guess_contains_node(lines):
    """Tries to determine if a file includes node
    """
    for line in lines:
        if "https://nodejs.org/download/release/v" in line:
            return 1
        if "(0) == (uv_async_init(uv_default_loop(), &dispatch_debug_messages_async, DispatchDebugMessagesAsyncCallback))" in line:
            return 1
        if "Documentation can be found at http://nodejs.org/" in line:
            return 1
    return 0

def get_version(lines, filename):
    """returns version information for node as found in a given file.
    The version info is returned as a tuple:
        [modulename, is_or_contains, version]

    modulename will be node if node is found (and blank otherwise)
    is_or_contains idicates if the file is a copy of node or contains one
    version gives the actual version number

    VPkg: nodejs, node.js
    """
    version_info = dict()
    if "bin/node" in filename:
        version_info["is_or_contains"] = "is"
    elif guess_contains_node(lines):
        version_info["is_or_contains"] = "contains"

    if "is_or_contains" in version_info:
        version_info["modulename"] = "node"
        version_info["version"] = guess_node_version_from_content(lines)

    return version_info




'''
Resource definitions and helper functions for tests.
'''
import sys
import shutil
import tempfile
import unittest
if sys.version_info.major == 3:
    from urllib.request import urlopen
else:
    from urllib2 import urlopen

CURL_7_20_0_RPM = 'curl-7.20.0-4.fc13.x86_64.rpm'
CURL_7_20_0_URL = 'https://archives.fedoraproject.org/pub/archive/fedora/linux/releases/13/Everything/x86_64/os/Packages/' + CURL_7_20_0_RPM
VMWARE_CAB = 'https://master.dl.sourceforge.net/project/winpe/VmWare%20Drivers/VmWare%20Drivers%20v1.1/vmware-1.1.cab'
TMUX_DEB = 'https://mirrors.cat.pdx.edu/ubuntu/pool/main/t/tmux/tmux_1.8-5_amd64.deb'

class TempDirTest(unittest.TestCase):
    """ For tests that need a temp directory """

    @classmethod
    def setUpClass(cls):
        cls.tempdir = tempfile.mkdtemp()

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.tempdir)

def download_file(url, target):
    download = urlopen(url)
    with open(target, 'wb') as target_file:
        target_file.write(download.read())
    download.close()



#!/usr/bin/python3

"""
CVE checker for libnss

References:
http://www.cvedetails.com/vulnerability-list/vendor_id-452/product_id-4052/Mozilla-Network-Security-Services.html

RSS feed: http://www.cvedetails.com/vulnerability-feed.php?vendor_id=452&product_id=4052&version_id=&orderby=3&cvssscoremin=0
"""
import re

def guess_nss_version_from_content(lines):
    """Guesses the libnss version from the file contents
    """
    new_guess = ""
    pattern1a = re.compile(r"\$Header: NSS ([01234]+\.[0-9]+) ")
    pattern1b = re.compile(r"\$Header: NSS ([01234]+\.[0-9]+\.[0-9]+) ")
    pattern1c = re.compile(r"\$Header: NSS ([01234]+\.[0-9]+\.[0-9]+)\.[0-9]+ ")
    pattern2a = re.compile(r"Version: NSS ([01234]+\.[0-9]+) Basic")
    pattern2b = re.compile(r"Version: NSS ([01234]+\.[0-9]+\.[0-9]+) Basic")
    pattern2c = re.compile(r"Version: NSS ([01234]+\.[0-9]+\.[0-9]+)\.[0-9]+ Basic")
    pattern3a = re.compile(r"\@\(#\)NSS ([01234]+\.[0-9]+) Basic")
    pattern3b = re.compile(r"\@\(#\)NSS ([01234]+\.[0-9]+\.[0-9]+) Basic")
    pattern3c = re.compile(r"\@\(#\)NSS ([01234]+\.[0-9]+\.[0-9]+)\.[0-9]+ Basic")

    for line in lines:
        match = pattern1a.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2
        match = pattern1b.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2
        match = pattern1c.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2
        match = pattern2a.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2

        match = pattern2b.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2
        match = pattern2c.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2
        match = pattern3a.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2

        match = pattern3b.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2
        match = pattern3c.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2

    return new_guess

def guess_contains_nss(lines):
    """Tries to determine if a file includes libnss
    """
    for line in lines:
        if "Client authentication failed: private key in key database does not match public key in certificate database." in line:
            return 1
        if "The next protocol negotiation extension was enabled, but the callback was cleared prior to being needed." in line:
            return 1
        if "# SSL/TLS secrets log file, generated by NSS" in line:
            return 1
    return 0

def get_version(lines, filename):
    """returns version information for libnss as found in a given file.
    The version info is returned as a tuple:
        [modulename, is_or_contains, version]

    modulename will be libnss if libnss is found (and blank otherwise)
    is_or_contains idicates if the file is a copy of libnss or contains one
    version gives the actual version number

    VPkg: mozilla, network_security_services
    """
    version_info = dict()
    if "libnss.so." in filename or "libssl3.so" in filename:
        version_info["is_or_contains"] = "is"
    elif guess_contains_nss(lines):
        version_info["is_or_contains"] = "contains"

    if "is_or_contains" in version_info:
        version_info["modulename"] = "nss"
        version_info["version"] = guess_nss_version_from_content(lines)

    return version_info




#!/usr/bin/python3

"""
CVE checker for curl CLI

References:
https://curl.haxx.se/docs/security.html
http://www.cvedetails.com/vulnerability-list/vendor_id-12682/Haxx.html

RSS feed: http://www.cvedetails.com/vulnerability-feed.php?vendor_id=12682&product_id=0&version_id=0&orderby=3&cvssscoremin=0

Note: Some of the "first vulnerable in" data may not be entered correctly.
"""
import re

def guess_curl_version_from_content(lines):
    """Guesses the curl version from the file contents
    """
    new_guess = ""
    pattern1 = re.compile(r"curl ([678]+\.[0-9]+\.[0-9]+)")

    for line in lines:
        match = pattern1.search(line)
        if match:
            new_guess2 = match.group(1).strip()
            if len(new_guess2) > len(new_guess):
                new_guess = new_guess2

    return new_guess

def get_version(lines, filename):
    """returns version information for curl as found in a given file.
    The version info is returned as a tuple:
        [modulename, is_or_contains, version]

    modulename will be curl if curl is found (and blank otherwise)
    is_or_contains idicates if the file is a copy of curl or contains one
    version gives the actual version number

    VPkg: haxx, curl
    """
    version_info = dict()
    if filename[::-1].startswith(("curl")[::-1]):
        version_info["is_or_contains"] = "is"

    if "is_or_contains" in version_info:
        version_info["modulename"] = "curl"
        version_info["version"] = guess_curl_version_from_content(lines)

    return version_info




import os
import csv
import json
import pickle
import shutil
import fnmatch
import collections

class DirWalk():
    """
    for filename in DirWalk('*.c').walk(roots):
        do a thing with the c-files in the roots directories
    """
    def __init__(self,
                 pattern='*',
                 folder_include_pattern='*',
                 folder_exclude_pattern='.git',
                 file_exclude_pattern='',
                 yield_files=True,
                 yield_folders=False):
        """
        Generator for walking the file system and filtering the results.
        """
        self.pattern = pattern
        self.folder_include_pattern = folder_include_pattern
        self.folder_exclude_pattern = folder_exclude_pattern
        self.file_exclude_pattern = file_exclude_pattern
        self.yield_files = yield_files
        self.yield_folders = yield_folders

    def walk(self, roots=None):
        """ Walk the directory looking for files matching the pattern """
        if roots is None:
            roots = []
        for root in roots:
            for dirpath, dirnames, filenames in os.walk(root):
                # Filters
                filenames[:] = [
                    i for i in filenames if self.pattern_match(i, self.pattern)
                    and not self.pattern_match(i, self.file_exclude_pattern)]
                dirnames[:] = [
                    i for i in dirnames
                    if self.pattern_match(i, self.folder_include_pattern)
                    and not self.pattern_match(i, self.folder_exclude_pattern)]
                # Yields
                if self.yield_files:
                    for filename in filenames:
                        yield os.path.normpath(os.path.join(dirpath, filename))
                if self.yield_folders:
                    for dirname in dirnames:
                        yield os.path.normpath(os.path.join(dirpath, dirname))

    @staticmethod
    def pattern_match(text, patterns):
        """ Match filename patterns """
        if not patterns:
            return False
        for pattern in patterns.split(';'):
            if fnmatch.fnmatch(text, pattern):
                return True
        return False



import ast
import os
from io import open
from setuptools import find_packages, setup

with open('README.md', 'r', encoding='utf-8') as f:
    readme = f.read()

setup(
    name='cve-bin-tool',
    version='0.2.0',
    description='CVE Binary Checker Tool',
    long_description=readme,
    author='Terri Oda',
    author_email='terri.oda@intel.com',
    maintainer='Terri Oda',
    maintainer_email='terri.oda@intel.com',
    url='https://github.com/intel/cve-bin-tool',
    license='GPLv3',
    keywords=[
        'security',
        'tools',
        'CVE',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'cve-bin-tool = cve_bin_tool.cli:main',
        ],
        'cve_bin_tool.checker': [
            '%s = cve_bin_tool.checkers.%s:get_version' \
                    % tuple((2 * [filename.replace('.py', '')])) \
            for filename in \
            os.listdir(os.path.join(os.path.abspath(os.path.dirname(__file__)),
                'cve_bin_tool', 'checkers')) \
                if filename[::-1].startswith('yp.') and \
                not '__init__' in filename
        ],
    },
)



'''
Extraction of archives
'''
import os
import shutil
import tempfile
import itertools
import subprocess
from contextlib import contextmanager

from .log import LOGGER

class ExtractionFailed(ValueError):
    pass

class UnknownArchiveType(ValueError):
    pass

@contextmanager
def PopenCTX(*args, **kwargs):
    ''' Python 2 does not support context manager style Popen.'''
    proc = subprocess.Popen(*args, **kwargs)
    try:
        yield proc
    finally:
        if 'stdout' in kwargs:
            proc.stdout.close()
        proc.terminate()
        proc.wait()

class BaseExtractor(object):
    '''Extracts tar, rpm, etc. files'''

    def __init__(self, logger=None):
        '''Sets up logger and if we should extract files or just report'''
        if logger is None:
            logger = LOGGER.getChild(self.__class__.__name__)
        self.logger = logger
        self.file_extractors = {
            self.extract_file_tar: ['.tgz', '.tar.gz', '.tar', '.tar.xz', '.tar.bz2'],
            self.extract_file_rpm: ['.rpm'],
            self.extract_file_deb: ['.deb', '.ipk'],
            self.extract_file_cab: ['.cab'],
            self.extract_file_zip: ['.exe', '.zip', '.jar'],
        }

    def can_extract(self, filename):
        """ Check if the filename is something we know how to extract """
        for extention in itertools.chain(*self.file_extractors.values()):
            if filename[::-1].startswith(extention[::-1]):
                return True
        return False

    @classmethod
    def extract_file_tar(cls, filename, extraction_path):
        """ Extract tar files """
        return subprocess.call(
            ["tar", "-C", extraction_path, "-axf", filename])

    @classmethod
    def extract_file_rpm(cls, filename, extraction_path):
        """ Extract rpm packages """
        with PopenCTX(["rpm2cpio", filename], stdout=subprocess.PIPE) as proc:
            return subprocess.call(["cpio", "-idmv"], stdin=proc.stdout,
                                   cwd=extraction_path)

    @classmethod
    def extract_file_deb(cls, filename, extraction_path):
        """ Extract debian packages """
        if subprocess.call(["ar", "x", filename], cwd=extraction_path) != 0:
            return 1
        for filename in os.listdir(extraction_path):
            if '.tar' in filename:
                result = subprocess.call(
                    ["tar", "-C", extraction_path, "-zxf",
                     os.path.join(extraction_path, filename)])
                os.unlink(os.path.join(extraction_path, filename))
                if result != 0:
                    return result
        return 0

    @classmethod
    def extract_file_cab(cls, filename, extraction_path):
        """ Extract cab files """
        return subprocess.call(
            ["cabextract", "-d", extraction_path, filename])

    @classmethod
    def extract_file_zip(cls, filename, extraction_path):
        """ Extract zip files """
        return subprocess.call(
            ["unzip", "-qq", "-n", "-d", extraction_path, filename])

class TempDirExtractorContext(BaseExtractor):
    '''Extracts tar, rpm, etc. files'''

    def __init__(self, raise_failure = False, *args, **kwargs):
        BaseExtractor.__init__(self, *args, **kwargs)
        self.tempdir = None
        self.raise_failure = raise_failure

    def extract(self, filename):
        """ Run the extractor """
        for extractor in self.file_extractors:
            for extention in self.file_extractors[extractor]:
                if filename[::-1].startswith(extention[::-1]):
                    extracted_path = os.path.join(self.tempdir,
                                                  os.path.basename(filename) + \
                                                  ".extracted")
                    if os.path.exists(extracted_path):
                        shutil.rmtree(extracted_path)
                    os.makedirs(extracted_path)
                    if extractor(filename, extracted_path) != 0:
                        if self.raise_failure:
                            raise ExtractionFailed(filename)
                        else:
                            self.logger.warning('Failure extracting %r',
                                                filename)
                    else:
                        self.logger.debug('Extracted %r to %r', filename,
                                          extracted_path)
                    return extracted_path
        raise UnknownArchiveType(filename)

    def __enter__(self):
        ''' Create a temporary directory to extract files to. '''
        self.tempdir = tempfile.mkdtemp()
        return self

    def __exit__(self, exc_type, exc, exc_tb):
        """ Removes all extraction directories that need to be cleaned up."""
        shutil.rmtree(self.tempdir)

class Extractor(BaseExtractor):
    '''Provides a context which extraction is done in'''

    def __call__(self, *args, **kwargs):
        return TempDirExtractorContext(*args, **kwargs)



import os
import sys
import shutil
import tarfile
import tempfile
import unittest
from zipfile import ZipFile, ZipInfo
from io import BytesIO
if sys.version_info.major == 3 and sys.version_info.minor >= 3:
    import lzma

from cve_bin_tool.extractor import Extractor

from .test_definitions import download_file, \
                              CURL_7_20_0_URL, \
                              VMWARE_CAB, \
                              TMUX_DEB

# Enable logging if tests are not passing to help you find errors
# import logging
# logging.basicConfig(level=logging.DEBUG)

class TestExtractorBase(unittest.TestCase):
    '''Test methods for extraction of various file types'''

    @classmethod
    def setUpClass(cls):
        cls.extractor = Extractor()
        cls.tempdir = tempfile.mkdtemp(prefix='cve-bin-tool-')

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.tempdir)

    def extract_files(self, filenames):
        # Make sure all test files are present
        for filename in filenames:
            self.assertTrue(os.path.exists(os.path.join(self.tempdir, filename)),
                            'test file %r was not found' % (filename,))
        # Make sure extraction reports success
        for filename in filenames:
            with self.extractor() as ectx:
                yield ectx.extract(os.path.join(self.tempdir, filename))

class TestExtractor(TestExtractorBase):

    def test_can_extract(self):
        self.assertTrue(self.extractor.can_extract('.tar.bz2'))
        self.assertTrue(self.extractor.can_extract('.zip'))
        self.assertTrue(self.extractor.can_extract('.deb'))

class TestExtractFileTar(TestExtractorBase):

    def setUp(self):
        for filename, tarmode in [('test.tgz', 'w:gz'),
                                  ('test.tar.gz', 'w:gz'),
                                  ('test.tar.bz2', 'w:bz2'),
                                  ('test.tar', 'w')]:
            tarpath = os.path.join(self.tempdir, filename)
            tar = tarfile.open(tarpath, mode=tarmode)
            data = 'feedface'.encode('utf-8')
            addfile = BytesIO(data)
            info = tarfile.TarInfo(name='test.txt')
            info.size = len(data)
            tar.addfile(tarinfo=info, fileobj=addfile)
            tar.close()
        if sys.version_info.major == 3 and sys.version_info.minor >= 3:
            tarpath = os.path.join(self.tempdir, 'test.tar')
            tarpath_xz = os.path.join(self.tempdir, 'test.tar.xz')
            with open(tarpath, 'rb') as infile, \
                    lzma.open(tarpath_xz , 'w') as outfile:
                outfile.write(infile.read())

    def test_extract_file_tar(self):
        try:
            self.extract_files(['test' + e for e in
                self.extractor.file_extractors[self.extractor.extract_file_tar]])
        except AssertionError as error:
            # Don't if we failed the xz test on versions were we couldn't create
            # the file
            if sys.version_info.major == 3 and sys.version_info.minor >= 3:
                raise

    def test_extract_cleanup(self):
        with self.extractor() as ectx:
            extracted_path = ectx.extract(os.path.join(self.tempdir,
                                                       'test.tar'))
            self.assertTrue(os.path.isdir(extracted_path))
        self.assertFalse(os.path.exists(extracted_path))

class TestExtractFileRpm(TestExtractorBase):

    def setUp(self):
        download_file(CURL_7_20_0_URL, os.path.join(self.tempdir, 'test.rpm'))

    def test_extract_file_rpm(self):
        for extracted_path in self.extract_files(['test' + e for e in
                self.extractor.file_extractors[self.extractor.extract_file_rpm]]):
            self.assertTrue(os.path.isfile(os.path.join(extracted_path, 'usr',
                                                        'bin', 'curl')))

class TestExtractFileDeb(TestExtractorBase):

    def setUp(self):
        download_file(TMUX_DEB, os.path.join(self.tempdir, 'test.deb'))
        shutil.copyfile(os.path.join(self.tempdir, 'test.deb'),
                        os.path.join(self.tempdir, 'test.ipk'))

    def test_extract_file_deb(self):
        for extracted_path in self.extract_files(['test' + e for e in
                self.extractor.file_extractors[self.extractor.extract_file_deb]]):
            self.assertTrue(os.path.isfile(os.path.join(extracted_path, 'usr',
                                                        'bin', 'tmux')))

class TestExtractFileCab(TestExtractorBase):

    def setUp(self):
        download_file(VMWARE_CAB, os.path.join(self.tempdir, 'test.cab'))

    def test_extract_file_cab(self):
        for extracted_path in self.extract_files(['test' + e for e in
                self.extractor.file_extractors[self.extractor.extract_file_cab]]):
            self.assertTrue(os.path.isfile(os.path.join(extracted_path,
                                                        'vmware.htm')))

class TestExtractFileZip(TestExtractorBase):

    def setUp(self):
        for filename in ['test.exe', 'test.zip', 'test.jar']:
            zippath = os.path.join(self.tempdir, filename)
            with ZipFile(zippath, 'w') as zipfile:
                zipfile.writestr(ZipInfo('test.txt'), 'feedface')

    def test_extract_file_zip(self):
        self.extract_files(['test' + e for e in
            self.extractor.file_extractors[self.extractor.extract_file_zip]])
